@@ -14,28 +14,26 @@
                 if ( $debug == 1 ) {
                   echo("DEBUG: Fetching data from cache. Expires in " . $conf['cachetime'] . " seconds.\n");
                 }
-                $index_array = g_cache_deserialize();
+                $index_array = g_cache_deserialize($index);
         }
     }
 
-    if ( ! isset($index_array) ) {
+    if (empty($index_array)) {
 
         if ( $debug == 1 ) {
             echo("DEBUG: Querying GMond for new data\n");
         }
         // Set up for cluster summary
-        $context = "index_array";
         include_once $conf['gweb_root'] . "/functions.php";
         include_once $conf['gweb_root'] . "/ganglia.php";
-        include_once $conf['gweb_root'] . "/get_ganglia.php";
+        $GLOBALS['context'] = "index_array";
+        include $conf['gweb_root'] . "/get_ganglia.php";
 
-        foreach ( $index_array['cluster'] as $hostname => $elements ) {
-            $hosts[] = $hostname;
+        // only save if the result looks good
+        if (count($index_array) > 0) {
+          $index_array['hosts'] = array_keys($index_array['cluster']);
+          g_cache_serialize($index_array);
         }
-        asort($hosts);
-        $index_array['hosts'] = $hosts;
-
-        g_cache_serialize($index_array);
     }
 
 ?>
